from typing import Optional, Union

from typing_extensions import TypeAlias

from dagster_shared.record import record
from dagster_shared.serdes import whitelist_for_serdes

DgDefinitionMetadata: TypeAlias = Union[
    "DgAssetMetadata",
    "DgSensorMetadata",
    "DgScheduleMetadata",
    "DgJobMetadata",
    "DgAssetCheckMetadata",
]


@whitelist_for_serdes
@record
class DgAssetMetadata:
    key: str
    deps: list[str]
    kinds: list[str]
    group: Optional[str]
    description: Optional[str]
    automation_condition: Optional[str]


@whitelist_for_serdes
@record
class DgSensorMetadata:
    name: str


@whitelist_for_serdes
@record
class DgScheduleMetadata:
    name: str
    cron_schedule: str


@whitelist_for_serdes
@record
class DgJobMetadata:
    name: str


@whitelist_for_serdes
@record
class DgAssetCheckMetadata:
    key: str
    asset_key: str
    name: str
    additional_deps: list[str]
    description: Optional[str]



import os
import sys
from pathlib import Path
from typing import Any

import tomlkit
import yaml


def is_windows() -> bool:
    return sys.platform == "win32"


def _get_default_dg_cli_file() -> Path:
    if is_windows():
        return Path.home() / "AppData" / "dg" / "dg.toml"
    else:
        return Path.home() / ".dg.toml"


DEFAULT_DG_CLI_FILE = _get_default_dg_cli_file()


def load_config(config_path: Path) -> dict[str, Any]:
    if config_path.suffix == ".toml":
        return tomlkit.parse(config_path.read_text()).unwrap()
    else:
        return yaml.safe_load(config_path.read_text()) or {}


def write_config(config_path: Path, config: dict[str, Any]):
    if config_path.suffix == ".toml":
        with open(config_path, "w") as f:
            tomlkit.dump(config, f)
    else:
        with open(config_path, "w") as f:
            yaml.safe_dump(config, f)


def get_dg_config_path() -> Path:
    return Path(os.getenv("DG_CLI_CONFIG", DEFAULT_DG_CLI_FILE))


def does_dg_config_file_exist() -> bool:
    return get_dg_config_path().exists()



import contextlib
import socket
from typing import TypeVar

T = TypeVar("T")


def find_free_port() -> int:
    with contextlib.closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
        s.bind(("", 0))
        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        return s.getsockname()[1]


def remove_none_recursively(obj: T) -> T:
    """Remove none values from a dict. This can be used to support comparing provided config vs.
    config we retrive from kubernetes, which returns all fields, even those which have no value
    configured.
    """
    if isinstance(obj, (list, tuple, set)):
        return type(obj)(remove_none_recursively(x) for x in obj if x is not None)
    elif isinstance(obj, dict):
        return type(obj)(
            (remove_none_recursively(k), remove_none_recursively(v))
            for k, v in obj.items()
            if k is not None and v is not None
        )
    else:
        return obj
